/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "stream.hpp"

#include "math/Vector3.hpp"

#include "utils/String.hpp"

template<>
void read<char>(DataInputStream& dis, char& v)
{
	v = dis.read();
}

template<>
void write<char>(DataOutputStream& dos, const char& v)
{
	dos.write(v);
}

template<>
void read<int>(DataInputStream& dis, int& v)
{
	v = dis.readInt();
}

template<>
void write<int>(DataOutputStream& dos, const int& v)
{
	dos.writeInt(v);
}

template<>
void read<float>(DataInputStream& dis, float& v)
{
	v = dis.readFloat();
}

template<>
void write<float>(DataOutputStream& dos, const float& v)
{
	dos.writeFloat(v);
}

template<>
void read<Vector3>(DataInputStream& dis, Vector3& v)
{
	read(dis, v.x);
	read(dis, v.y);
	read(dis, v.z);
}

template<>
void write<Vector3>(DataOutputStream& dos, const Vector3& v)
{
	write(dos, v.x);
	write(dos, v.y);
	write(dos, v.z);
}

template<>
void read<String>(DataInputStream& dis, String& v)
{
	int len;
	
	read(dis, len);
	
	char* string = new char[len + 1];
	
	for (int i = 0; i < len; i++)
	{
		read(dis, string[i]);
	}
	string[len] = 0;
	
	v = string;
}

template<>
void write<String>(DataOutputStream& dos, const String& v)
{
	int len = v.length();
	write(dos, len);
	
	for (int i = 0; i < len; i++)
	{
		write(dos, v[i]);
	}
}
